from example_app.models import Todo
from datetime import datetime, date

actions = ['create', 'started', 'updated', 'done']
titles = [
    'do something',
    'do something else',
    'other stuff',
    'groceries',
    'have lunch',
]
for title in titles:
    Todo.objects.create(title=title)


from .streamio import get_client, StreamObject
from datetime import date, datetime


class StreamModelMixin:
    def get_stream_object(self):
        return StreamObject(self)

    def track_action(
        self,
        verb,
        by=None,
        create_collection=True,
        force_update=False,
        time=None,
        date_field=None,
        extra_data=None,
        custom_message=None,
    ):
        """
        # minimal:
        todo.track_action('finish')
        """
        stream = StreamObject(self)
        enriched = None
        if create_collection:
            enriched = stream.enrich(force_update=force_update)

        if by is None:
            by = getattr(self, self.feed_actor_field)

        is_onceoff_action = verb in self.feed_once_off_actions

        if (
            custom_message is None
            and getattr(self, "formatted_feed_message", None) is not None
        ):
            custom_message = self.formatted_feed_message(verb=verb)

        activity = stream.perform_action(
            by,
            verb,
            is_onceoff_action=is_onceoff_action,
            custom_message=custom_message,
            date_field=date_field,
            time=time,
            extra_data=extra_data,
        )
        return {"object": enriched, "activity": activity}

    def add_notification(
        self, verb, message, users_to_notify=None, forward={}, *args, **kwargs
    ):
        if users_to_notify is None:
            actor_id = getattr(self, self.feed_actor_field, None)
            if actor_id is not None:
                users_to_notify = [actor_id]

        stream = StreamObject(self)
        return stream.add_notification(
            users_to_notify=users_to_notify, verb=verb, message=message, forward=forward
        )

    def add_reaction(
        self, activity_kind, activity_id, data=None, by=None, target_feeds=None
    ):
        if data is None:
            data = {}
        stream = StreamObject(self)
        if by is None:
            by = str(getattr(self, self.feed_actor_field))
        if target_feeds is None:
            target_feeds = stream.get_to_feeds()
        # max limiit on target feeds is 20
        if target_feeds and len(target_feeds) > 19:
            target_feeds = target_feeds[:19]

        return stream.client.reactions.add(
            activity_kind, activity_id, user_id=by, data=data, target_feeds=target_feeds
        )

    def get_reactions(self, activity_id, activity_kind=None):
        stream = StreamObject(self)
        data = {
            "activity_id": activity_id,
        }
        if activity_kind is not None:
            data["kind"] = activity_kind

        return stream.client.reactions.filter(**data)

    def get_feed(self, **kwargs):
        options = {"enrich": True}
        options.update(kwargs)
        client = get_client()
        return client.feed(self.feed_name, self.pk).get(**options)



from rest_framework.decorators import action
from rest_framework.response import Response
from .helpers import stream_view, streamactivity_view

class StreamViewSetMixin:
    """
    A mixin for using with DRF viewsets.
    Provides a GET /:resource/:id/stream/ endpoint

    """

    @action(detail=True, methods=['GET'])
    def stream(self, request, pk=None):
        return stream_view(self, request, pk=None)

    @action(detail=True, methods=['GET'])
    def streamactivity(self, request, pk=None):
        return streamactivity_view(self, request, pk=None)


from rest_framework.response import Response

def stream_view(self, request, pk=None):
    options_fields = ['limit', 'offset', 'id_gt', 'id_gte', 'id_lt', 'id_lte', 'offset', 'ranking', 'enrich']
    extra_args = {}
    for field in options_fields:
        value = request.GET.get(field, None)
        if value is not None:
            extra_args[field] = value
            if field == 'enrich':
                true_values = ['true', 'True', 't', '1', 1]
                if value in true_values:
                    extra_args[field] = True
                else:
                    extra_args[field] = False

    obj = self.get_object()
    stream = obj.get_feed(**extra_args)
    token = obj.get_stream_object().client.create_user_session_token(obj.pk)
    stream.update({
        "livestream_token": token
    })
    return Response(stream)

def streamactivity_view(self, request, pk=None):

    feed = self.get_object().get_feed(**{"limit": 100, "enrich": False})

    activities = feed.get('results', [])
    action_breakdown = {}
    timeseries = { "total": {} }
    to_time = None
    from_time = None

    if activities:
        to_time = activities[0].get('time')
        from_time = activities[len(activities) - 1].get('time')

        for activity in activities:
            object_type = activity.get('object').split(":")[1]
            verb = activity.get('verb')
            key = "{}:{}".format(object_type, verb)
            count = action_breakdown.get(key, 0)
            action_breakdown[key] = (count + 1)

            day = activity.get('time').date().isoformat()
            count = timeseries.get("total").get(day, 0)
            key_count = timeseries.get(key, {}).get(day, 0)

            timeseries["total"][day] = (count + 1)
            if not (key in timeseries):
                timeseries[key] = {}
            timeseries[key][day] = (key_count + 1)


    return Response({
        "period": {
            "from": from_time,
            "to": to_time
        },
        "count": len(activities),
        # stacked timeseries:
        "timeseries": timeseries,
        "radial": action_breakdown
    })